<p align="center">
  <a href="https://github.com/daynin/fundoc">
    <img src="./images/logo.png" height="200px"/>
  </a>
</p>

<h2 align="center">
    Language agnostic documentation generator. Written in Rust 🦀
</h2>

<p align="center">
  <a href="https://travis-ci.org/daynin/fundoc">
    <img alt="Tests Status" src="https://github.com/daynin/fundoc/workflows/tests/badge.svg">
  </a>
  <a href="https://github.com/daynin/fundoc/blob/master/LICENSE">
    <img alt="License" src="https://img.shields.io/badge/license-MIT-blue.svg">
  </a>
  <a href="https://github.com/daynin/fundoc/issues">
    <img alt="GitHub Issues" src="https://img.shields.io/github/issues/daynin/fundoc.svg">
  </a>
  <a href="https://crates.io/crates/fundoc">
    <img alt="Crates.io" src="https://img.shields.io/crates/v/fundoc.svg">
  </a>
</p>

**Fundoc** - the right way to generate documentation. Business features in your project can be implemented in different files and even in different technologies. Fundoc can merge all descriptions of those features and put them in appropriate files.

Fundoc's main goals:

- Allow you to keep all your **documentation along with your code**. Separating documentation and code makes it harder to support projects.
- Use **the same versioning tools for your documentation** and code. All versions of your documentation should match versions of source code otherwise we can't trust documentation.
- Documentation generators should allow you to **write your documentation from different file types** like source code files (Rust, C++, TypeScript, Java, JavaScript, Ruby, Python, etc), specification files (Alloy, TLA+, etc), stylesheet files (CSS, SCSS, QT Stylesheets, etc), configs (JSON, TOML, YAML, etc).

### Why you should use it

If you want to keep your documentation synchronized with your code and to be able to describe different parts of a business feature in places where it was implemented you can write documentation in your source files or near them.

#### One source file, one documentation file

You can describe business logic in one particular source file and generate documentation from it. It can be useful for describing config files, features implemented in one file or component, etc. Here you can find [an example](https://github.com/CSSSR/linters) of using Fundoc to describe ESLint/Prettier configs.

<p align="center">
	<img src="./images/example-1.png" />
</p>

#### Multiple source files, one documentation file

You can write your documentation in multiple source files using different programming languages and still have an ability to get all documentation in one file (one file per article).

<p align="center">
	<img src="./images/example-2.png" />
</p>

Also you can use .md files combining documentation from source files and from `.fdoc.md` files. In that case the name of an article will be got from a name of a file.

It can help you keep your documentation up to date and change only that part of the documentation that is connected to the code you're changing. And your team members can easily find the source of a documentation part using generated links on source files.

By the way, [documentation](https://csssr.github.io/fundoc/) here is generated by Fundoc itself, so you can check out how it works.

### Installation

You can install Fundoc by executing this shell script:

```bash
curl -LSfs https://japaric.github.io/trust/install.sh | \
    sh -s -- --git daynin/fundoc
```

If you're rust developer and you have had rust compiler installed, you can install Fundoc from [crates.io](https://crates.io/):

```bash
cargo install fundoc
```

Or manually download it from the [releases page](https://github.com/daynin/fundoc/releases).

### How to use

You can create a config file by executing `fundoc --init` command and answering few questions:

<p align="center">
    <img src="./images/fundoc-init.png" />
</p>

All Fundoc's parameters are described [here](https://csssr.github.io/fundoc/configuration.html)

To generate documentation run `fundoc`:

<p align="center">
    <img src="./images/fundoc-work.png" />
</p>

Fundoc supports [mdBook](https://github.com/rust-lang/mdBook), so if you want to generate documentation in mdBook format you can do it by setting `mdBook: true` in your config file. It can be useful if you want to use GitHub Pages with full-text search support in your project.

<p align="center">
    <img src="./images/full-text-search.gif" />
</p>

### Plugins

You can write your own plugins to enhance Fundoc's abilities. To do so, you can simply write a Lua plugin like this:

```lua
function transform(text)
  importMermaid = '<script type="module"> import mermaid from "https://cdn.jsdelivr.net/npm/mermaid@10.0.2/+esm"; mermaid.initialize({}); </script>'

  result = '<pre class="mermaid">' .. text .. '</pre>' .. importMermaid
e
```

[Read more](https://daynin.github.io/fundoc/plugins.html) about plugins!

### Companies/projects who use Fundoc

- [Space Twistie](https://github.com/Space-Twistie)
- [CSSSR](https://csssr.com/en)
- [Zaya Tracker](https://github.com/Zaya-Tracker)

Know of others? Create a PR to let me know!

### GitHub Action

You can use [Fundoc's GitHub Action](https://github.com/daynin/fundoc-action) to automate documentation generation.

### Roadmap

- [x] Merging documentation from different files.
- [x] Different file types support.
- [x] Creating links to the source files from the documentation.
- [x] Generating mdBook with an ability to search through documentation.
- [x] Multi-repositories support (collecting all documentation fragments from different repos).
- [x] Add Mermaid support
- [x] Add a plugin-system
- [ ] Add file system API for the plugin system
- [ ] Add FFI for the plugin system
- [ ] Zettelkasten method support (maybe)

